import unittest
import json
from timer import Timer
import time

class TestSelfTest(unittest.TestCase):
    def __init__(self, *args):
        super().__init__(*args)
        pass

    def setUp(self):
        pass

    def test_connect1(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652()
        self.assertTrue(s.connected)

    def test_selfTest(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652()
        self.assertTrue(s.connected)

        s['powerAccel'] = 'lownoise'
        s['powerGyro'] = 'lownoise'

        s['accelFullScale'] = 16
        s['gyroFullScale'] = 2000

        s['accelODR'] = 1000
        s['gyroODR'] = 1000

        time.sleep(50e-3)

        aResp1i = s['accelDataInt']
        gResp1i = s['gyroDataInt']
        aResp1 = s['accelData']
        gResp1 = s['gyroData']

        s['SELF_TEST_CONFIG'] = 0x7F
        time.sleep(50e-3)

        aResp2 = s['accelData']
        gResp2 = s['gyroData']
        aResp2i = s['accelDataInt']
        gResp2i = s['gyroDataInt']
        s['SELF_TEST_CONFIG'] = 0

        aTest  = s['accelSelfTestData']
        gTest  = s['gyroSelfTestData']

        astResp = [ aResp2i[i] - aResp1i[i] for i in range(3) ]
        gstResp = [ gResp2i[i] - gResp1i[i] for i in range(3) ]
        print(astResp, aTest)
        print(gstResp, gTest)

        astRespErr = [ abs(1e-1 * astResp[i] - aTest[i]) for i in range(3) ]
        gstRespErr = [ abs(1e-1 * gstResp[i] - gTest[i]) for i in range(3) ]
        print(astRespErr, aTest)
        print(gstRespErr, gTest)

        astRespRelErr = [ 1e2 * astRespErr[i] / aResp2i[i] for i in range(3) ]
        gstRespRelErr = [ 1e2 * gstRespErr[i] / gResp2i[i] for i in range(3) ]
        print(f'accel self test rel. dev. (%): {astRespRelErr}')
        print(f'gyro self test rel. dev. (%): {gstRespRelErr}')

        self.assertTrue(not any([ astRespRelErr[i] > 5 for i in range(3) ]))
        self.assertTrue(not any([ gstRespRelErr[i] > 5 for i in range(3) ]))

    def test_accel_selfTest(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652()
        self.assertTrue(s.connected)

        res = s.accelSelfTest()
        print('accel self test rel err %', res[2])
        self.assertTrue(res[0])
        self.assertTrue(not any([ res[3][i] > 5 for i in range(3) ]))

    def test_gyro_selfTest(self):
        from iim42652_spi import iim42652
        s = iim42652.getIIM42652()
        self.assertTrue(s.connected)

        res = s.gyroSelfTest()
        print('gyro self test rel err %', res[2])
        self.assertTrue(res[0])
        self.assertTrue(not any([ res[3][i] > 5 for i in range(3) ]))
